import logging

from lyricsfinder import Lyrics
from lyricsfinder.extractor import LyricsExtractor
from lyricsfinder.utils import Request, clean_lyrics

log = logging.getLogger(__name__)


class LyricalNonsense(LyricsExtractor):
    name = "Lyrical Nonsense"
    url = "http://www.lyrical-nonsense.com/"
    display_url = "lyrical-nonsense.com"

    @classmethod
    async def extract_lyrics(cls, url_data: Request) -> Lyrics:
        bs = await url_data.bs
        title_el = bs.select_one("span.titletext2new")
        if not title_el:
            title_el = bs.select_one("div.titlelyricblocknew h1")
        title = title_el.text

        artist = bs.select_one("div.artistcontainer span.artisttext2new")
        if not artist:
            artist = bs.select_one("div.artistcontainer h2")

        artist = artist.text

        lyrics_window = bs.select_one("div#Romaji div.olyrictext") or bs.select_one("div#Lyrics div.olyrictext")
        text = "\n\n".join(tag.text for tag in lyrics_window.find_all("p"))
        lyrics = clean_lyrics(text)

        return Lyrics(title, lyrics, artist=artist)



import logging
import re
from typing import AsyncIterator, NamedTuple

from aiohttp import ClientResponse, ClientSession
from bs4 import BeautifulSoup

log = logging.getLogger(__name__)


class Request:
    def __init__(self, session: ClientSession, url: str):
        self.session = session

        self._url = url
        self.resp_kwargs = {}
        self.headers = {}

        self._resp = None
        self._text = None
        self._bs = None

    def __repr__(self) -> str:
        """Return string rep."""
        return "<{}>".format(self.url)

    @property
    def url(self) -> str:
        return self._url

    @url.setter
    def url(self, value: str):
        self._url = value
        self._resp = None
        self._text = None
        self._bs = None

    @property
    async def resp(self) -> ClientResponse:
        if not self._resp:
            self._resp = await self.session.get(self.url, headers=self.headers, **self.resp_kwargs)
        return self._resp

    @property
    async def text(self) -> str:
        if not self._text:
            self._text = await (await self.resp).text()
        return self._text

    @property
    async def bs(self) -> BeautifulSoup:
        if not self._bs:
            self._bs = BeautifulSoup(await self.text, "lxml")
        return self._bs


class GoogleSearchItem(NamedTuple):
    link: str


async def google_search(session: ClientSession, query: str, api_key: str) -> AsyncIterator[GoogleSearchItem]:
    item_count = 10

    params = {
        "q": query,
        "key": api_key,
        "cx": "002017775112634544492:7y5bpl2sn78",
        "fields": "items(link)",
        "num": item_count
    }

    for start in range(1, 101, item_count):
        params["start"] = start
        log.debug(f"getting search results starting from {start}")
        async with session.get("https://www.googleapis.com/customsearch/v1", params=params) as resp:
            data = await resp.json()
            items = data.get("items", [])
            for item in items:
                yield GoogleSearchItem(**item)


RE_CHAR_COLLAPSERS = [
    (r"～", "~"),
    (r"[“”]", "\"")
]


def clean_lyrics(lyrics: str, *, allowed: str = "") -> str:
    lyrics = lyrics.strip()
    for target, replacement in RE_CHAR_COLLAPSERS:
        lyrics = re.sub(target, replacement, lyrics)

    lyrics = re.sub(rf"[^\w\[\]()/ \"',.:\-~\n?!{allowed}]+", "", lyrics)  # remove unwanted characters
    lyrics = re.sub(r" +", " ", lyrics)  # reduce to one space only
    lyrics = re.sub(r"\n{2,}", "\n\n", lyrics)  # reduce to max 2 new lines in a row
    lyrics = re.sub(r" +?\n", "\n", lyrics)  # remove space before newline

    return lyrics



import abc
import logging
from typing import TYPE_CHECKING

from .utils import Request

if TYPE_CHECKING:
    from .lyrics_manager import Lyrics

log = logging.getLogger(__name__)


class LyricsExtractorMount(type):
    """Registers new Extractors."""

    def __init__(cls, *args):
        super().__init__(*args)

        if not hasattr(cls, "extractors"):
            cls.extractors = []
            log.debug("Created Extractor Meta Class")
        else:
            cls.extractors.append(cls)
            log.debug(f"Registered {cls}")

    def __str__(self):
        """Return string rep."""
        return "<Extractor {}>".format(self.__name__)


class LyricsExtractor(metaclass=LyricsExtractorMount):
    name: str
    url: str
    display_url: str

    @classmethod
    async def can_handle(cls, url_data: Request) -> bool:
        """Check whether this extractor can extract lyrics from this url."""
        return cls.display_url in url_data.url

    @classmethod
    @abc.abstractmethod
    async def extract_lyrics(cls, url_data: Request) -> "Lyrics":
        """Return a Lyrics object for the given url, text or bs."""
        raise NotImplementedError



import importlib
import os


def load_extractors():
    for module in os.listdir(os.path.dirname(__file__)):
        if module == "__init__.py" or module[-3:] != ".py":
            continue

        name = ".{}".format(module[:-3])
        importlib.import_module(name, __name__)



import logging
from datetime import datetime

from lyricsfinder import Lyrics, NoLyrics
from lyricsfinder.extractor import LyricsExtractor
from lyricsfinder.utils import Request

log = logging.getLogger(__name__)


class Genius(LyricsExtractor):
    name = "Genius"
    url = "https://genius.com/"
    display_url = "genius.com"

    @classmethod
    async def extract_lyrics(cls, request: Request) -> Lyrics:
        bs = await request.bs

        lyrics_window = bs.find_all("div", {"class": "lyrics"})
        if not lyrics_window:
            raise NoLyrics

        lyrics_window = lyrics_window[0]
        lyrics = lyrics_window.text.strip()

        title = bs.find("h1", attrs={"class": "header_with_cover_art-primary_info-title"}).text
        artist = bs.select_one("a.header_with_cover_art-primary_info-primary_artist").string
        release_date = None
        date_str = bs.find(text="Release Date")
        if date_str:
            date_str = date_str.parent.find_next_sibling("span").string
            release_date = datetime.strptime(date_str, "%B %d, %Y")

        return Lyrics(title, lyrics, artist=artist, release_date=release_date)



from datetime import datetime
from typing import Any, Dict


class LyricsOrigin:

    def __init__(self, url: str, source_name: str, source_url: str, *, query: str = None):
        self.url = url
        self.source_name = source_name
        self.source_url = source_url
        self.query = query

    def __str__(self) -> str:
        return self.source_name

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "LyricsOrigin":
        return cls(**data)

    def to_dict(self) -> Dict[str, Any]:
        return {
            "query": self.query,
            "url": self.url,
            "source_name": self.source_name,
            "source_url": self.source_url
        }


class Lyrics:
    def __init__(self, title: str, lyrics: str, artist: str = None, release_date: datetime = None, *, origin: LyricsOrigin = None):
        self.title = title
        self.artist = artist
        self.release_date = release_date
        self.lyrics = lyrics
        self.origin = origin

    def __str__(self) -> str:
        return "<Lyrics for \"{}\" from {}>".format(self.title, self.origin)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]):
        data["origin"] = LyricsOrigin.from_dict(data["origin"])
        if data["release_date"]:
            data["release_date"] = datetime.fromtimestamp(data["release_date"])
        return cls(**data)

    def set_origin(self, origin: LyricsOrigin):
        self.origin = origin

    def to_dict(self) -> Dict[str, Any]:
        return {
            "title": self.title,
            "artist": self.artist,
            "release_date": self.release_date.timestamp() if self.release_date else None,
            "lyrics": self.lyrics,
            "origin": self.origin.to_dict()
        }



import logging

from .__info__ import *
from .lyrics_manager import LyricsManager
from .models import *

log = logging.getLogger(__name__)

extract_lyrics = LyricsManager.extract_lyrics
search_lyrics = LyricsManager.search_lyrics

LyricsManager.setup()
log.debug("LyricsFinder loaded")



import logging
import re
from typing import Pattern

from lyricsfinder import Lyrics
from lyricsfinder.extractor import LyricsExtractor
from lyricsfinder.utils import Request

log = logging.getLogger(__name__)

RE_SPLIT_ARTIST_TITLE: Pattern = re.compile(r"(?P<artist>.+?)\s+–\s+(?P<title>.+) (?:lyrics)?")


class Lyricsmode(LyricsExtractor):
    name = "Lyricsmode"
    url = "http://www.lyricsmode.com/"
    display_url = "lyricsmode.com"

    @classmethod
    async def extract_lyrics(cls, request: Request) -> Lyrics:
        bs = await request.bs
        lyrics_window = bs.find_all("p", {"id": "lyrics_text", "class": "ui-annotatable"})[0]
        lyrics = lyrics_window.text.strip()

        heading = bs.find("h1", attrs={"class": "song_name fs32"}).text.strip()

        match = RE_SPLIT_ARTIST_TITLE.search(heading)

        artist = match.group("artist")
        title = match.group("title")

        return Lyrics(title, lyrics, artist)



import logging
import re

from lyricsfinder import Lyrics, NoLyrics
from lyricsfinder.extractor import LyricsExtractor
from lyricsfinder.utils import Request

log = logging.getLogger(__name__)


class AZLyrics(LyricsExtractor):
    name = "AZLyrics"
    url = "https://www.azlyrics.com/"
    display_url = "azlyrics.com"

    @classmethod
    async def extract_lyrics(cls, request: Request) -> Lyrics:
        request.headers = {"user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:59.0) Gecko/20100101 Firefox/59.0"}
        bs = await request.bs

        center = bs.body.find("div", {"class": "col-xs-12 col-lg-8 text-center"})
        if not center:
            raise NoLyrics

        lyrics = center.find("div", {"class": None}).text

        lyrics = re.sub(r"<br>", " ", lyrics)
        lyrics = re.sub(r"<i?>\W*", "[", lyrics)
        lyrics = re.sub(r"\W*</i>", "]", lyrics)
        lyrics = re.sub(r"&quot;", "\"", lyrics)
        lyrics = re.sub(r"</div>", "", lyrics)

        title = center.find("h1").text.strip()[1:-8]
        artist = bs.select_one("div.lyricsh h2 b").string[:-7]
        lyrics = lyrics.strip()

        return Lyrics(title, lyrics, artist=artist)



LyricsException = type("LyricsException", (Exception,), {})
NoLyrics = type("NoLyrics", (LyricsException,), {})
NotAllowedError = type("NotAllowedError", (LyricsException,), {})


class NoExtractorError(LyricsException):
    def __init__(self, url: str):
        super().__init__(f"No extractor found for {url}")



#!/usr/bin/env python

import asyncio
import json
import textwrap
from argparse import ArgumentParser, Namespace
from pathlib import Path
from typing import Any

import lyricsfinder

_DEFAULT = object()

config_location = Path.home() / ".lyricsfinder"
_config = None


def load_config():
    global _config

    if not config_location.is_file():
        raise FileNotFoundError("No .lyricsfinder file found in home directory!")

    try:
        data = json.loads(config_location.read_text("utf-8"))
    except json.JSONDecodeError:
        config_location.unlink()
        raise TypeError("Couldn't parse lyricsfinder config...")
    else:
        if not isinstance(data, dict):
            raise TypeError("Wrong data type stored in config file")

        _config = data


def save_config():
    config_location.write_text(json.dumps(_config), "utf-8")


def config_get(key: str, default=_DEFAULT) -> Any:
    if not _config:
        try:
            load_config()
        except FileNotFoundError:
            if default is _DEFAULT:
                raise
            return default

    value = _config.get(key, default)
    if value is _DEFAULT:
        raise KeyError(f"Config doesn't have key {key}")
    return value


def config_set(key: str, value: Any, *, flush=True):
    global _config

    if not _config:
        try:
            load_config()
        except FileNotFoundError:
            _config = {}

    _config[key] = value
    if flush:
        save_config()


def print_lyrics(lyrics: lyricsfinder.Lyrics):
    if not lyrics:
        return

    title = lyrics.title or "Unknown"
    artist = lyrics.artist or "Unknown"

    header = f"{title} by {artist}"
    if lyrics.release_date:
        header += f" ({lyrics.release_date.year})"

    width = max(len(header), 40)
    header = header.center(width)
    line = width * "="

    lyrics_text = textwrap.fill(lyrics.lyrics, width, replace_whitespace=False, drop_whitespace=False)

    text = f"{header}\n" \
           f"{line}\n" \
           f"{lyrics_text}\n" \
           f"\n" \
           f"from {lyrics.origin.source_name}"

    print(text)


def search(args: Namespace):
    api_key = args.token
    if api_key is None:
        api_key = config_get("google_api_key", None)
        if not api_key:
            raise ValueError("No API key specified, and none saved!")
    else:
        config_set("google_api_key", api_key)

    query = " ".join(args.query)
    lyrics = asyncio.run(lyricsfinder.search_lyrics(query, api_key=api_key).__anext__())
    print_lyrics(lyrics)


def extract(args: Namespace):
    lyrics = asyncio.run(lyricsfinder.extract_lyrics(args.url))
    print_lyrics(lyrics)


def main(*args):
    args = args or None

    parser = ArgumentParser("lyricsfinder", description="Find the lyrics you've always wanted to find")
    subparsers = parser.add_subparsers()

    search_parser = subparsers.add_parser("search")
    search_parser.set_defaults(func=search)
    search_parser.add_argument("query", help="Query to search for", nargs="+")
    search_parser.add_argument("-t", "--token", help="Google Search API key")

    extract_parser = subparsers.add_parser("extract")
    extract_parser.set_defaults(func=extract)
    extract_parser.add_argument("url", help="Url to extract lyrics from")

    args = parser.parse_args(args)

    if hasattr(args, "func"):
        args.func(args)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



import logging
import re
from datetime import datetime

from aiohttp import ClientResponseError

from lyricsfinder import Lyrics
from lyricsfinder.extractor import LyricsExtractor
from lyricsfinder.utils import Request
from ..models import exceptions

log = logging.getLogger(__name__)

ORDINAL_MATCHER = re.compile(r"(\d{1,2})(st|nd|rd|th)")


class MusixMatch(LyricsExtractor):
    name = "MusixMatch"
    url = "https://www.musixmatch.com/"
    display_url = "musixmatch.com"

    @classmethod
    async def extract_lyrics(cls, request: Request) -> Lyrics:
        request.headers = {"user-agent": "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:40.0) Gecko/20100101 Firefox/40.1"}
        try:
            (await request.resp).raise_for_status()
        except ClientResponseError:
            raise exceptions.NotAllowedError
        bs = await request.bs

        if bs.find_all("div", attrs={"class": "mxm-empty-state", "data-reactid": "87"}):
            raise exceptions.NoLyrics

        lyrics_frame = bs.find_all("div", {"class": "mxm-lyrics"})

        if not lyrics_frame:
            raise exceptions.NoLyrics

        lyrics_window = lyrics_frame[0].find_all("div", {"class": "mxm-lyrics"})

        if not lyrics_window:
            raise exceptions.NoLyrics

        lyrics_window = lyrics_window[0].span

        for garbage in bs.find_all("script"):
            garbage.replace_with(2 * "\n")

        lyrics = lyrics_window.text
        title = bs.find("h1", attrs={"class": "mxm-track-title__track"}).contents[-1].strip()
        artist = bs.select_one("a.mxm-track-title__artist").string
        release_date = None
        date_str = bs.select_one("div.mxm-track-footer__album h3.mui-cell__subtitle")
        if date_str:
            date_str = ORDINAL_MATCHER.sub(lambda m: m.group(1).zfill(2), date_str.string)
            release_date = datetime.strptime(date_str, "%b %d %Y")

        return Lyrics(title, lyrics, artist=artist, release_date=release_date)



import logging
from contextlib import asynccontextmanager
from typing import AsyncIterator, List

from aiohttp import ClientSession

from . import extractors
from .extractor import LyricsExtractor
from .models import Lyrics, LyricsOrigin, exceptions
from .utils import Request, google_search

log = logging.getLogger(__name__)


class LyricsManager:
    extractors: List[LyricsExtractor] = []

    @classmethod
    def setup(cls):
        """Load extractors."""
        log.debug("setting up")
        extractors.load_extractors()

        cls.extractors = LyricsExtractor.extractors
        log.info("loaded {} extractors".format(len(cls.extractors)))

    @classmethod
    @asynccontextmanager
    async def get_session(cls, session: ClientSession = None) -> ClientSession:
        if session:
            yield session
        else:
            async with ClientSession() as session:
                yield session

    @classmethod
    async def extract_lyrics(cls, url: str, *, session: ClientSession = None) -> Lyrics:
        async with cls.get_session(session) as session:
            exc = None

            log.info("extracting lyrics from url \"{url}\"")
            request = Request(session, url)

            for extractor in cls.extractors:
                if not await extractor.can_handle(request):
                    continue

                log.debug(f"using {extractor} for {request}")
                try:
                    lyrics = await extractor.extract_lyrics(request)
                except exceptions.NoLyrics:
                    log.warning(f"{extractor} didn't find any lyrics at {url}")
                    continue
                except exceptions.NotAllowedError:
                    log.warning(f"{extractor} couldn't access lyrics at {url}")
                    continue
                except Exception as e:
                    e.__cause__ = exc
                    exc = e
                    log.exception(f"Something went wrong when {extractor} handled {request}")
                    continue
                else:
                    lyrics.set_origin(LyricsOrigin(url, extractor.name, extractor.url))
                    log.debug(f"extracted lyrics {lyrics}")
                    return lyrics

            raise exceptions.NoExtractorError(url) from exc

    @classmethod
    async def search_lyrics(cls, query: str, *, api_key: str, session: ClientSession = None) -> AsyncIterator[Lyrics]:

        async with cls.get_session(session) as session:
            result_iter = google_search(session, query, api_key)

            async for result in result_iter:
                url = result.link
                try:
                    lyrics = await cls.extract_lyrics(url)
                except exceptions.NoExtractorError:
                    log.warning(f"No extractor for url {url}")
                    continue
                else:
                    lyrics.origin.query = query
                    yield lyrics

        log.warning("No lyrics found for query \"{}\"".format(query))



import logging
import re

from lyricsfinder import Lyrics, NoLyrics
from lyricsfinder.extractor import LyricsExtractor
from lyricsfinder.utils import Request

log = logging.getLogger(__name__)

ARTIST_MATCHER = re.compile(r"^Performed by ([\w' ]+)\b", re.MULTILINE)


class Animelyrics(LyricsExtractor):
    name = "Animelyrics"
    url = "http://www.animelyrics.com/"
    display_url = "animelyrics.com"

    @classmethod
    async def extract_lyrics(cls, request: Request) -> Lyrics:
        bs = await request.bs
        title = next(bs.select_one("div ~ h1").children).string.strip()
        artist = bs.find(text=ARTIST_MATCHER)
        if artist:
            artist = ARTIST_MATCHER.match(artist).group(1)

        lyrics_window = bs.find("table", attrs={"cellspacing": "0", "border": "0"})

        if lyrics_window:  # shit's been translated
            log.info("these lyrics have been translated... sighs...")

            lines = lyrics_window.find_all("tr")
            lyrics = ""
            for line in lines:
                p = line.td
                if p:
                    p.span.dt.replace_with("")
                    for br in p.span.find_all("br"):
                        br.replace_with("\n")

                    lyrics += p.span.text
            lyrics = lyrics.strip()
        else:
            text_url = re.sub(r"\.html?", ".txt", request.url)
            request.url = text_url
            request.resp_kwargs["allow_redirects"] = False
            text = await request.text
            content = text.strip()
            match = re.search(r"-{10,}(.+?)-{10,}", content, flags=re.DOTALL)
            if match:
                lyrics = match.group(1).strip()
            else:
                raise NoLyrics

        lyrics = lyrics.replace("\xa0", " ").replace("\r", "")

        return Lyrics(title, lyrics, artist=artist)
